@@ -33,13 +33,12 @@
 
 #include "kadmin_locl.h"
 
-RCSID("$Id$");
-
 static char *check_library  = NULL;
 static char *check_function = NULL;
 static getarg_strings policy_libraries = { 0, NULL };
 static char *config_file;
-static char *keytab_str = "HDB:";
+static char sHDB[] = "HDB:";
+static char *keytab_str = sHDB;
 static int help_flag;
 static int version_flag;
 static int debug_flag;
@@ -67,12 +66,12 @@ static struct getargs args[] = {
       "password check function to load", "function" },
 #endif
     {	"debug",	'd',	arg_flag,   &debug_flag,
-	"enable debugging"
+	"enable debugging", NULL
     },
     {	"ports",	'p',	arg_string, &port_str,
 	"ports to listen to", "port" },
-    {	"help",		'h',	arg_flag,   &help_flag },
-    {	"version",	'v',	arg_flag,   &version_flag }
+    {	"help",		'h',	arg_flag,   &help_flag, NULL, NULL },
+    {	"version",	'v',	arg_flag,   &version_flag, NULL, NULL }
 };
 
 static int num_args = sizeof(args) / sizeof(args[0]);
@@ -95,6 +94,7 @@ main(int argc, char **argv)
     int i;
     krb5_log_facility *logfacility;
     krb5_keytab keytab;
+    krb5_socket_t sfd = rk_INVALID_SOCKET;
 
     setprogname(argv[0]);
 
@@ -162,14 +162,18 @@ main(int argc, char **argv)
 
     if(debug_flag) {
 	int debug_port;
-	
+
 	if(port_str == NULL)
 	    debug_port = krb5_getportbyname (context, "kerberos-adm",
 					     "tcp", 749);
 	else
 	    debug_port = htons(atoi(port_str));
-	mini_inetd(debug_port);
+	mini_inetd(debug_port, &sfd);
     } else {
+#ifdef _WIN32
+	pidfile(NULL);
+	start_server(context, port_str);
+#else
 	struct sockaddr_storage __ss;
 	struct sockaddr *sa = (struct sockaddr *)&__ss;
 	socklen_t sa_size = sizeof(__ss);
@@ -178,19 +182,20 @@ main(int argc, char **argv)
 	 * Check if we are running inside inetd or not, if not, start
 	 * our own server.
 	 */
-	
+
 	if(roken_getsockname(STDIN_FILENO, sa, &sa_size) < 0 &&
-	       errno == ENOTSOCK) {
-	    parse_ports(context, port_str ? port_str : "+");
+	   rk_SOCK_ERRNO == ENOTSOCK) {
 	    pidfile(NULL);
-	    start_server(context);
+	    start_server(context, port_str);
 	}
+#endif /* _WIN32 */
+	sfd = STDIN_FILENO;
     }
-    
+
     if(realm)
 	krb5_set_default_realm(context, realm); /* XXX */
 
-    kadmind_loop(context, keytab, STDIN_FILENO);
+    kadmind_loop(context, keytab, sfd);
 
     return 0;
 }
