@@ -276,6 +276,10 @@ struct string_list {
     struct string_list *next;
 };
 
+/* Declarations for Bison */
+#define YYMALLOC malloc
+#define YYFREE   free
+
 
 
 /* Enabling traces.  */
@@ -298,7 +302,7 @@ struct string_list {
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 67 "asn1parse.y"
+#line 71 "asn1parse.y"
 {
     int constant;
     struct value *value;
@@ -314,7 +318,7 @@ typedef union YYSTYPE
     struct constraint_spec *constraint_spec;
 }
 /* Line 193 of yacc.c.  */
-#line 318 "asn1parse.c"
+#line 322 "asn1parse.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -327,7 +331,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 331 "asn1parse.c"
+#line 335 "asn1parse.c"
 
 #ifdef short
 # undef short
@@ -670,21 +674,21 @@ static const yytype_int16 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   235,   235,   242,   243,   245,   247,   250,   252,   255,
-     256,   259,   260,   263,   264,   267,   268,   271,   283,   289,
-     290,   293,   294,   297,   298,   301,   307,   315,   325,   326,
-     327,   330,   331,   332,   333,   334,   335,   336,   337,   338,
-     339,   340,   341,   342,   343,   346,   353,   363,   371,   379,
-     390,   395,   401,   409,   415,   420,   424,   437,   445,   448,
-     455,   463,   469,   478,   486,   487,   492,   498,   506,   515,
-     521,   529,   537,   544,   545,   548,   559,   564,   571,   587,
-     593,   596,   597,   600,   606,   614,   624,   630,   643,   652,
-     655,   659,   663,   670,   673,   677,   684,   695,   698,   703,
-     708,   713,   718,   723,   728,   733,   741,   747,   752,   763,
-     774,   780,   786,   794,   800,   807,   820,   821,   824,   831,
-     834,   845,   849,   860,   866,   867,   870,   871,   872,   873,
-     874,   877,   880,   883,   894,   902,   908,   916,   924,   927,
-     932
+       0,   239,   239,   246,   247,   249,   251,   254,   256,   259,
+     260,   263,   264,   267,   268,   271,   272,   275,   287,   293,
+     294,   297,   298,   301,   302,   305,   311,   319,   329,   330,
+     331,   334,   335,   336,   337,   338,   339,   340,   341,   342,
+     343,   344,   345,   346,   347,   350,   357,   367,   375,   383,
+     394,   399,   405,   413,   419,   424,   428,   441,   449,   452,
+     459,   467,   473,   482,   490,   491,   496,   502,   510,   519,
+     525,   533,   541,   548,   549,   552,   563,   568,   575,   591,
+     597,   600,   601,   604,   610,   618,   628,   634,   647,   656,
+     659,   663,   667,   674,   677,   681,   688,   699,   702,   707,
+     712,   717,   722,   727,   732,   737,   745,   751,   756,   767,
+     778,   784,   790,   798,   804,   811,   824,   825,   828,   835,
+     838,   849,   853,   864,   870,   871,   874,   875,   876,   877,
+     878,   881,   884,   887,   898,   906,   912,   920,   928,   931,
+     936
 };
 #endif
 
@@ -1770,29 +1774,29 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 237 "asn1parse.y"
+#line 241 "asn1parse.y"
     {
 			checkundefined();
 		}
     break;
 
   case 4:
-#line 244 "asn1parse.y"
-    { error_message("implicit tagging is not supported"); }
+#line 248 "asn1parse.y"
+    { lex_error_message("implicit tagging is not supported"); }
     break;
 
   case 5:
-#line 246 "asn1parse.y"
-    { error_message("automatic tagging is not supported"); }
+#line 250 "asn1parse.y"
+    { lex_error_message("automatic tagging is not supported"); }
     break;
 
   case 7:
-#line 251 "asn1parse.y"
-    { error_message("no extensibility options supported"); }
+#line 255 "asn1parse.y"
+    { lex_error_message("no extensibility options supported"); }
     break;
 
   case 17:
-#line 272 "asn1parse.y"
+#line 276 "asn1parse.y"
     {
 		    struct string_list *sl;
 		    for(sl = (yyvsp[(1) - (4)].sl); sl != NULL; sl = sl->next) {
@@ -1805,7 +1809,7 @@ yyreduce:
     break;
 
   case 18:
-#line 284 "asn1parse.y"
+#line 288 "asn1parse.y"
     {
 		    struct string_list *sl;
 		    for(sl = (yyvsp[(2) - (3)].sl); sl != NULL; sl = sl->next)
@@ -1814,7 +1818,7 @@ yyreduce:
     break;
 
   case 25:
-#line 302 "asn1parse.y"
+#line 306 "asn1parse.y"
     {
 		    (yyval.sl) = emalloc(sizeof(*(yyval.sl)));
 		    (yyval.sl)->string = (yyvsp[(1) - (3)].name);
@@ -1823,7 +1827,7 @@ yyreduce:
     break;
 
   case 26:
-#line 308 "asn1parse.y"
+#line 312 "asn1parse.y"
     {
 		    (yyval.sl) = emalloc(sizeof(*(yyval.sl)));
 		    (yyval.sl)->string = (yyvsp[(1) - (1)].name);
@@ -1832,7 +1836,7 @@ yyreduce:
     break;
 
   case 27:
-#line 316 "asn1parse.y"
+#line 320 "asn1parse.y"
     {
 		    Symbol *s = addsym ((yyvsp[(1) - (3)].name));
 		    s->stype = Stype;
@@ -1843,7 +1847,7 @@ yyreduce:
     break;
 
   case 45:
-#line 347 "asn1parse.y"
+#line 351 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Boolean,
 				     TE_EXPLICIT, new_type(TBoolean));
@@ -1851,12 +1855,12 @@ yyreduce:
     break;
 
   case 46:
-#line 354 "asn1parse.y"
+#line 358 "asn1parse.y"
     {
 		    if((yyvsp[(2) - (5)].value)->type != integervalue)
-			error_message("Non-integer used in first part of range");
+			lex_error_message("Non-integer used in first part of range");
 		    if((yyvsp[(2) - (5)].value)->type != integervalue)
-			error_message("Non-integer in second part of range");
+			lex_error_message("Non-integer in second part of range");
 		    (yyval.range) = ecalloc(1, sizeof(*(yyval.range)));
 		    (yyval.range)->min = (yyvsp[(2) - (5)].value)->u.integervalue;
 		    (yyval.range)->max = (yyvsp[(4) - (5)].value)->u.integervalue;
@@ -1864,10 +1868,10 @@ yyreduce:
     break;
 
   case 47:
-#line 364 "asn1parse.y"
-    {	
+#line 368 "asn1parse.y"
+    {
 		    if((yyvsp[(2) - (5)].value)->type != integervalue)
-			error_message("Non-integer in first part of range");
+			lex_error_message("Non-integer in first part of range");
 		    (yyval.range) = ecalloc(1, sizeof(*(yyval.range)));
 		    (yyval.range)->min = (yyvsp[(2) - (5)].value)->u.integervalue;
 		    (yyval.range)->max = (yyvsp[(2) - (5)].value)->u.integervalue - 1;
@@ -1875,10 +1879,10 @@ yyreduce:
     break;
 
   case 48:
-#line 372 "asn1parse.y"
-    {	
+#line 376 "asn1parse.y"
+    {
 		    if((yyvsp[(4) - (5)].value)->type != integervalue)
-			error_message("Non-integer in second part of range");
+			lex_error_message("Non-integer in second part of range");
 		    (yyval.range) = ecalloc(1, sizeof(*(yyval.range)));
 		    (yyval.range)->min = (yyvsp[(4) - (5)].value)->u.integervalue + 2;
 		    (yyval.range)->max = (yyvsp[(4) - (5)].value)->u.integervalue;
@@ -1886,10 +1890,10 @@ yyreduce:
     break;
 
   case 49:
-#line 380 "asn1parse.y"
+#line 384 "asn1parse.y"
     {
 		    if((yyvsp[(2) - (3)].value)->type != integervalue)
-			error_message("Non-integer used in limit");
+			lex_error_message("Non-integer used in limit");
 		    (yyval.range) = ecalloc(1, sizeof(*(yyval.range)));
 		    (yyval.range)->min = (yyvsp[(2) - (3)].value)->u.integervalue;
 		    (yyval.range)->max = (yyvsp[(2) - (3)].value)->u.integervalue;
@@ -1897,7 +1901,7 @@ yyreduce:
     break;
 
   case 50:
-#line 391 "asn1parse.y"
+#line 395 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Integer,
 				     TE_EXPLICIT, new_type(TInteger));
@@ -1905,7 +1909,7 @@ yyreduce:
     break;
 
   case 51:
-#line 396 "asn1parse.y"
+#line 400 "asn1parse.y"
     {
 			(yyval.type) = new_type(TInteger);
 			(yyval.type)->range = (yyvsp[(2) - (2)].range);
@@ -1914,7 +1918,7 @@ yyreduce:
     break;
 
   case 52:
-#line 402 "asn1parse.y"
+#line 406 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TInteger);
 		  (yyval.type)->members = (yyvsp[(3) - (4)].members);
@@ -1923,7 +1927,7 @@ yyreduce:
     break;
 
   case 53:
-#line 410 "asn1parse.y"
+#line 414 "asn1parse.y"
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -1932,7 +1936,7 @@ yyreduce:
     break;
 
   case 54:
-#line 416 "asn1parse.y"
+#line 420 "asn1parse.y"
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[(1) - (3)].members), (yyvsp[(3) - (3)].member), members);
 			(yyval.members) = (yyvsp[(1) - (3)].members);
@@ -1940,12 +1944,12 @@ yyreduce:
     break;
 
   case 55:
-#line 421 "asn1parse.y"
+#line 425 "asn1parse.y"
     { (yyval.members) = (yyvsp[(1) - (3)].members); }
     break;
 
   case 56:
-#line 425 "asn1parse.y"
+#line 429 "asn1parse.y"
     {
 			(yyval.member) = emalloc(sizeof(*(yyval.member)));
 			(yyval.member)->name = (yyvsp[(1) - (4)].name);
@@ -1959,7 +1963,7 @@ yyreduce:
     break;
 
   case 57:
-#line 438 "asn1parse.y"
+#line 442 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TInteger);
 		  (yyval.type)->members = (yyvsp[(3) - (4)].members);
@@ -1968,7 +1972,7 @@ yyreduce:
     break;
 
   case 59:
-#line 449 "asn1parse.y"
+#line 453 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TBitString);
 		  (yyval.type)->members = emalloc(sizeof(*(yyval.type)->members));
@@ -1978,7 +1982,7 @@ yyreduce:
     break;
 
   case 60:
-#line 456 "asn1parse.y"
+#line 460 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TBitString);
 		  (yyval.type)->members = (yyvsp[(4) - (5)].members);
@@ -1987,7 +1991,7 @@ yyreduce:
     break;
 
   case 61:
-#line 464 "asn1parse.y"
+#line 468 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_OID,
 				     TE_EXPLICIT, new_type(TOID));
@@ -1995,7 +1999,7 @@ yyreduce:
     break;
 
   case 62:
-#line 470 "asn1parse.y"
+#line 474 "asn1parse.y"
     {
 		    Type *t = new_type(TOctetString);
 		    t->range = (yyvsp[(3) - (3)].range);
@@ -2005,7 +2009,7 @@ yyreduce:
     break;
 
   case 63:
-#line 479 "asn1parse.y"
+#line 483 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Null,
 				     TE_EXPLICIT, new_type(TNull));
@@ -2013,17 +2017,17 @@ yyreduce:
     break;
 
   case 64:
-#line 486 "asn1parse.y"
+#line 490 "asn1parse.y"
     { (yyval.range) = NULL; }
     break;
 
   case 65:
-#line 488 "asn1parse.y"
+#line 492 "asn1parse.y"
     { (yyval.range) = (yyvsp[(2) - (2)].range); }
     break;
 
   case 66:
-#line 493 "asn1parse.y"
+#line 497 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSequence);
 		  (yyval.type)->members = (yyvsp[(3) - (4)].members);
@@ -2032,7 +2036,7 @@ yyreduce:
     break;
 
   case 67:
-#line 499 "asn1parse.y"
+#line 503 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSequence);
 		  (yyval.type)->members = NULL;
@@ -2041,7 +2045,7 @@ yyreduce:
     break;
 
   case 68:
-#line 507 "asn1parse.y"
+#line 511 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSequenceOf);
 		  (yyval.type)->range = (yyvsp[(2) - (4)].range);
@@ -2051,7 +2055,7 @@ yyreduce:
     break;
 
   case 69:
-#line 516 "asn1parse.y"
+#line 520 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSet);
 		  (yyval.type)->members = (yyvsp[(3) - (4)].members);
@@ -2060,7 +2064,7 @@ yyreduce:
     break;
 
   case 70:
-#line 522 "asn1parse.y"
+#line 526 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSet);
 		  (yyval.type)->members = NULL;
@@ -2069,7 +2073,7 @@ yyreduce:
     break;
 
   case 71:
-#line 530 "asn1parse.y"
+#line 534 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TSetOf);
 		  (yyval.type)->subtype = (yyvsp[(3) - (3)].type);
@@ -2078,7 +2082,7 @@ yyreduce:
     break;
 
   case 72:
-#line 538 "asn1parse.y"
+#line 542 "asn1parse.y"
     {
 		  (yyval.type) = new_type(TChoice);
 		  (yyval.type)->members = (yyvsp[(3) - (4)].members);
@@ -2086,19 +2090,19 @@ yyreduce:
     break;
 
   case 75:
-#line 549 "asn1parse.y"
+#line 553 "asn1parse.y"
     {
 		  Symbol *s = addsym((yyvsp[(1) - (1)].name));
 		  (yyval.type) = new_type(TType);
 		  if(s->stype != Stype && s->stype != SUndefined)
-		    error_message ("%s is not a type\n", (yyvsp[(1) - (1)].name));
+		    lex_error_message ("%s is not a type\n", (yyvsp[(1) - (1)].name));
 		  else
 		    (yyval.type)->symbol = s;
 		}
     break;
 
   case 76:
-#line 560 "asn1parse.y"
+#line 564 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_GeneralizedTime,
 				     TE_EXPLICIT, new_type(TGeneralizedTime));
@@ -2106,7 +2110,7 @@ yyreduce:
     break;
 
   case 77:
-#line 565 "asn1parse.y"
+#line 569 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UTCTime,
 				     TE_EXPLICIT, new_type(TUTCTime));
@@ -2114,7 +2118,7 @@ yyreduce:
     break;
 
   case 78:
-#line 572 "asn1parse.y"
+#line 576 "asn1parse.y"
     {
 		    /* if (Constraint.type == contentConstrant) {
 		       assert(Constraint.u.constraint.type == octetstring|bitstring-w/o-NamedBitList); // remember to check type reference too
@@ -2130,14 +2134,14 @@ yyreduce:
     break;
 
   case 79:
-#line 588 "asn1parse.y"
+#line 592 "asn1parse.y"
     {
 		    (yyval.constraint_spec) = (yyvsp[(2) - (3)].constraint_spec);
 		}
     break;
 
   case 83:
-#line 601 "asn1parse.y"
+#line 605 "asn1parse.y"
     {
 		    (yyval.constraint_spec) = new_constraint_spec(CT_CONTENTS);
 		    (yyval.constraint_spec)->u.content.type = (yyvsp[(2) - (2)].type);
@@ -2146,10 +2150,10 @@ yyreduce:
     break;
 
   case 84:
-#line 607 "asn1parse.y"
+#line 611 "asn1parse.y"
     {
 		    if ((yyvsp[(3) - (3)].value)->type != objectidentifiervalue)
-			error_message("Non-OID used in ENCODED BY constraint");
+			lex_error_message("Non-OID used in ENCODED BY constraint");
 		    (yyval.constraint_spec) = new_constraint_spec(CT_CONTENTS);
 		    (yyval.constraint_spec)->u.content.type = NULL;
 		    (yyval.constraint_spec)->u.content.encoding = (yyvsp[(3) - (3)].value);
@@ -2157,10 +2161,10 @@ yyreduce:
     break;
 
   case 85:
-#line 615 "asn1parse.y"
+#line 619 "asn1parse.y"
     {
 		    if ((yyvsp[(5) - (5)].value)->type != objectidentifiervalue)
-			error_message("Non-OID used in ENCODED BY constraint");
+			lex_error_message("Non-OID used in ENCODED BY constraint");
 		    (yyval.constraint_spec) = new_constraint_spec(CT_CONTENTS);
 		    (yyval.constraint_spec)->u.content.type = (yyvsp[(2) - (5)].type);
 		    (yyval.constraint_spec)->u.content.encoding = (yyvsp[(5) - (5)].value);
@@ -2168,14 +2172,14 @@ yyreduce:
     break;
 
   case 86:
-#line 625 "asn1parse.y"
+#line 629 "asn1parse.y"
     {
 		    (yyval.constraint_spec) = new_constraint_spec(CT_USER);
 		}
     break;
 
   case 87:
-#line 631 "asn1parse.y"
+#line 635 "asn1parse.y"
     {
 			(yyval.type) = new_type(TTag);
 			(yyval.type)->tag = (yyvsp[(1) - (3)].tag);
@@ -2189,7 +2193,7 @@ yyreduce:
     break;
 
   case 88:
-#line 644 "asn1parse.y"
+#line 648 "asn1parse.y"
     {
 			(yyval.tag).tagclass = (yyvsp[(2) - (4)].constant);
 			(yyval.tag).tagvalue = (yyvsp[(3) - (4)].constant);
@@ -2198,56 +2202,56 @@ yyreduce:
     break;
 
   case 89:
-#line 652 "asn1parse.y"
+#line 656 "asn1parse.y"
     {
 			(yyval.constant) = ASN1_C_CONTEXT;
 		}
     break;
 
   case 90:
-#line 656 "asn1parse.y"
+#line 660 "asn1parse.y"
     {
 			(yyval.constant) = ASN1_C_UNIV;
 		}
     break;
 
   case 91:
-#line 660 "asn1parse.y"
+#line 664 "asn1parse.y"
     {
 			(yyval.constant) = ASN1_C_APPL;
 		}
     break;
 
   case 92:
-#line 664 "asn1parse.y"
+#line 668 "asn1parse.y"
     {
 			(yyval.constant) = ASN1_C_PRIVATE;
 		}
     break;
 
   case 93:
-#line 670 "asn1parse.y"
-    {
-			(yyval.constant) = TE_EXPLICIT;
-		}
-    break;
-
-  case 94:
 #line 674 "asn1parse.y"
     {
 			(yyval.constant) = TE_EXPLICIT;
 		}
     break;
 
-  case 95:
+  case 94:
 #line 678 "asn1parse.y"
+    {
+			(yyval.constant) = TE_EXPLICIT;
+		}
+    break;
+
+  case 95:
+#line 682 "asn1parse.y"
     {
 			(yyval.constant) = TE_IMPLICIT;
 		}
     break;
 
   case 96:
-#line 685 "asn1parse.y"
+#line 689 "asn1parse.y"
     {
 			Symbol *s;
 			s = addsym ((yyvsp[(1) - (4)].name));
@@ -2259,7 +2263,7 @@ yyreduce:
     break;
 
   case 98:
-#line 699 "asn1parse.y"
+#line 703 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_GeneralString,
 				     TE_EXPLICIT, new_type(TGeneralString));
@@ -2267,7 +2271,7 @@ yyreduce:
     break;
 
   case 99:
-#line 704 "asn1parse.y"
+#line 708 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_TeletexString,
 				     TE_EXPLICIT, new_type(TTeletexString));
@@ -2275,7 +2279,7 @@ yyreduce:
     break;
 
   case 100:
-#line 709 "asn1parse.y"
+#line 713 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UTF8String,
 				     TE_EXPLICIT, new_type(TUTF8String));
@@ -2283,7 +2287,7 @@ yyreduce:
     break;
 
   case 101:
-#line 714 "asn1parse.y"
+#line 718 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_PrintableString,
 				     TE_EXPLICIT, new_type(TPrintableString));
@@ -2291,7 +2295,7 @@ yyreduce:
     break;
 
   case 102:
-#line 719 "asn1parse.y"
+#line 723 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_VisibleString,
 				     TE_EXPLICIT, new_type(TVisibleString));
@@ -2299,7 +2303,7 @@ yyreduce:
     break;
 
   case 103:
-#line 724 "asn1parse.y"
+#line 728 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_IA5String,
 				     TE_EXPLICIT, new_type(TIA5String));
@@ -2307,7 +2311,7 @@ yyreduce:
     break;
 
   case 104:
-#line 729 "asn1parse.y"
+#line 733 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_BMPString,
 				     TE_EXPLICIT, new_type(TBMPString));
@@ -2315,7 +2319,7 @@ yyreduce:
     break;
 
   case 105:
-#line 734 "asn1parse.y"
+#line 738 "asn1parse.y"
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UniversalString,
 				     TE_EXPLICIT, new_type(TUniversalString));
@@ -2323,7 +2327,7 @@ yyreduce:
     break;
 
   case 106:
-#line 742 "asn1parse.y"
+#line 746 "asn1parse.y"
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -2332,7 +2336,7 @@ yyreduce:
     break;
 
   case 107:
-#line 748 "asn1parse.y"
+#line 752 "asn1parse.y"
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[(1) - (3)].members), (yyvsp[(3) - (3)].member), members);
 			(yyval.members) = (yyvsp[(1) - (3)].members);
@@ -2340,7 +2344,7 @@ yyreduce:
     break;
 
   case 108:
-#line 753 "asn1parse.y"
+#line 757 "asn1parse.y"
     {
 		        struct member *m = ecalloc(1, sizeof(*m));
 			m->name = estrdup("...");
@@ -2352,7 +2356,7 @@ yyreduce:
     break;
 
   case 109:
-#line 764 "asn1parse.y"
+#line 768 "asn1parse.y"
     {
 		  (yyval.member) = emalloc(sizeof(*(yyval.member)));
 		  (yyval.member)->name = (yyvsp[(1) - (2)].name);
@@ -2364,7 +2368,7 @@ yyreduce:
     break;
 
   case 110:
-#line 775 "asn1parse.y"
+#line 779 "asn1parse.y"
     {
 			(yyval.member) = (yyvsp[(1) - (1)].member);
 			(yyval.member)->optional = 0;
@@ -2373,7 +2377,7 @@ yyreduce:
     break;
 
   case 111:
-#line 781 "asn1parse.y"
+#line 785 "asn1parse.y"
     {
 			(yyval.member) = (yyvsp[(1) - (2)].member);
 			(yyval.member)->optional = 1;
@@ -2382,7 +2386,7 @@ yyreduce:
     break;
 
   case 112:
-#line 787 "asn1parse.y"
+#line 791 "asn1parse.y"
     {
 			(yyval.member) = (yyvsp[(1) - (3)].member);
 			(yyval.member)->optional = 0;
@@ -2391,7 +2395,7 @@ yyreduce:
     break;
 
   case 113:
-#line 795 "asn1parse.y"
+#line 799 "asn1parse.y"
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -2400,7 +2404,7 @@ yyreduce:
     break;
 
   case 114:
-#line 801 "asn1parse.y"
+#line 805 "asn1parse.y"
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[(1) - (3)].members), (yyvsp[(3) - (3)].member), members);
 			(yyval.members) = (yyvsp[(1) - (3)].members);
@@ -2408,7 +2412,7 @@ yyreduce:
     break;
 
   case 115:
-#line 808 "asn1parse.y"
+#line 812 "asn1parse.y"
     {
 		  (yyval.member) = emalloc(sizeof(*(yyval.member)));
 		  (yyval.member)->name = (yyvsp[(1) - (4)].name);
@@ -2422,26 +2426,26 @@ yyreduce:
     break;
 
   case 117:
-#line 821 "asn1parse.y"
+#line 825 "asn1parse.y"
     { (yyval.objid) = NULL; }
     break;
 
   case 118:
-#line 825 "asn1parse.y"
+#line 829 "asn1parse.y"
     {
 			(yyval.objid) = (yyvsp[(2) - (3)].objid);
 		}
     break;
 
   case 119:
-#line 831 "asn1parse.y"
+#line 835 "asn1parse.y"
     {
 			(yyval.objid) = NULL;
 		}
     break;
 
   case 120:
-#line 835 "asn1parse.y"
+#line 839 "asn1parse.y"
     {
 		        if ((yyvsp[(2) - (2)].objid)) {
 				(yyval.objid) = (yyvsp[(2) - (2)].objid);
@@ -2453,19 +2457,19 @@ yyreduce:
     break;
 
   case 121:
-#line 846 "asn1parse.y"
+#line 850 "asn1parse.y"
     {
 			(yyval.objid) = new_objid((yyvsp[(1) - (4)].name), (yyvsp[(3) - (4)].constant));
 		}
     break;
 
   case 122:
-#line 850 "asn1parse.y"
+#line 854 "asn1parse.y"
     {
 		    Symbol *s = addsym((yyvsp[(1) - (1)].name));
 		    if(s->stype != SValue ||
 		       s->value->type != objectidentifiervalue) {
-			error_message("%s is not an object identifier\n",
+			lex_error_message("%s is not an object identifier\n",
 				      s->name);
 			exit(1);
 		    }
@@ -2474,18 +2478,18 @@ yyreduce:
     break;
 
   case 123:
-#line 861 "asn1parse.y"
+#line 865 "asn1parse.y"
     {
 		    (yyval.objid) = new_objid(NULL, (yyvsp[(1) - (1)].constant));
 		}
     break;
 
   case 133:
-#line 884 "asn1parse.y"
+#line 888 "asn1parse.y"
     {
 			Symbol *s = addsym((yyvsp[(1) - (1)].name));
 			if(s->stype != SValue)
-				error_message ("%s is not a value\n",
+				lex_error_message ("%s is not a value\n",
 						s->name);
 			else
 				(yyval.value) = s->value;
@@ -2493,7 +2497,7 @@ yyreduce:
     break;
 
   case 134:
-#line 895 "asn1parse.y"
+#line 899 "asn1parse.y"
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = stringvalue;
@@ -2502,7 +2506,7 @@ yyreduce:
     break;
 
   case 135:
-#line 903 "asn1parse.y"
+#line 907 "asn1parse.y"
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = booleanvalue;
@@ -2511,7 +2515,7 @@ yyreduce:
     break;
 
   case 136:
-#line 909 "asn1parse.y"
+#line 913 "asn1parse.y"
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = booleanvalue;
@@ -2520,7 +2524,7 @@ yyreduce:
     break;
 
   case 137:
-#line 917 "asn1parse.y"
+#line 921 "asn1parse.y"
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = integervalue;
@@ -2529,13 +2533,13 @@ yyreduce:
     break;
 
   case 139:
-#line 928 "asn1parse.y"
+#line 932 "asn1parse.y"
     {
 		}
     break;
 
   case 140:
-#line 933 "asn1parse.y"
+#line 937 "asn1parse.y"
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = objectidentifiervalue;
@@ -2545,7 +2549,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2549 "asn1parse.c"
+#line 2553 "asn1parse.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2759,13 +2763,13 @@ yyreturn:
 }
 
 
-#line 940 "asn1parse.y"
+#line 944 "asn1parse.y"
 
 
 void
 yyerror (const char *s)
 {
-     error_message ("%s\n", s);
+     lex_error_message ("%s\n", s);
 }
 
 static Type *
